/** 
 * 
 * Copyright 2004 Hiram Chirino
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.activemq.store.journal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import javax.jms.JMSException;

import org.activeio.journal.RecordLocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.activemq.message.ActiveMQMessage;
import org.codehaus.activemq.message.MessageAck;
import org.codehaus.activemq.service.MessageIdentity;
import org.codehaus.activemq.service.QueueMessageContainer;
import org.codehaus.activemq.store.MessageStore;
import org.codehaus.activemq.store.cache.CacheMessageStore;
import org.codehaus.activemq.store.cache.CacheMessageStoreAware;
import org.codehaus.activemq.util.Callback;
import org.codehaus.activemq.util.TransactionTemplate;

/**
 * A MessageStore that uses a Journal to store it's messages.
 * 
 * @version $Revision: 1.5 $
 */
public class JournalMessageStore implements MessageStore, CacheMessageStoreAware {

	private static final Log log = LogFactory.getLog(JournalMessageStore.class);
	
	protected final JournalPersistenceAdapter peristenceAdapter;
	protected final MessageStore longTermStore;
	protected final String destinationName;
	protected final TransactionTemplate transactionTemplate;
	
	private RecordLocation nextMark;	
	private HashMap addedMessageIds = new HashMap();
	private ArrayList removedMessageLocations = new ArrayList();
	
	/** A MessageStore that we can use to retreive messages quickly. */
	private MessageStore cacheMessageStore = this;
	
	private boolean sync = true;

	public JournalMessageStore(JournalPersistenceAdapter adapter, MessageStore checkpointStore, String destinationName, boolean sync) {
		this.peristenceAdapter = adapter;
		this.longTermStore = checkpointStore;
		this.destinationName = destinationName;
		this.sync=sync;
		this.transactionTemplate = new TransactionTemplate(adapter);
	}

	/**
	 * Not synchronized since the Journal has better throughput if you increase
	 * the number of conncurrent writes that it is doing.
	 */
	public void addMessage(ActiveMQMessage message) throws JMSException {
		boolean sync = message.isReceiptRequired();
		RecordLocation location = peristenceAdapter.writePacket(destinationName, message, sync);
		synchronized(this) {
			nextMark = location;
			addedMessageIds.put(message.getJMSMessageIdentity(), location);
		}
	}	
	
	/**
	 */
	public void removeMessage(MessageAck ack)
			throws JMSException {

		RecordLocation ackLocation = peristenceAdapter.writePacket(destinationName, ack, sync);

		synchronized(this) {
			nextMark = ackLocation;
			if( addedMessageIds.remove(ack.getMessageIdentity())==null ) {
				removedMessageLocations.add(ack);
			}
		}		
	}

	/**
	 * @return
	 * @throws JMSException
	 */
	public RecordLocation checkpoint() throws JMSException {
	    
		RecordLocation rc;		
		final HashMap addedMessageIds;
		final ArrayList removedMessageLocations;
		
		// swap out the message hashmaps..
		synchronized(this) {		    
		    rc = nextMark;
		    addedMessageIds = this.addedMessageIds;
			removedMessageLocations = this.removedMessageLocations;
			
			this.nextMark = null;
			this.addedMessageIds = new HashMap();
			this.removedMessageLocations = new ArrayList();
		}
		
		transactionTemplate.run(new Callback() {
			public void execute() throws Throwable {
				
				// Checkpoint the added messages.
				Iterator iterator = addedMessageIds.keySet().iterator();
				while (iterator.hasNext()) {					
					MessageIdentity identity = (MessageIdentity) iterator.next();					
					ActiveMQMessage msg = getCacheMessage(identity);
					// Pull it out of the journal if we have to.
					if( msg==null ) {
					    RecordLocation location = (RecordLocation) addedMessageIds.get(identity);
						msg = (ActiveMQMessage) peristenceAdapter.readPacket((RecordLocation)location);
					}					
					longTermStore.addMessage(msg);
				}				
				
				// Checkpoint the removed messages.
				iterator = removedMessageLocations.iterator();
				while (iterator.hasNext()) {					
				    MessageAck ack = (MessageAck)iterator.next();
					longTermStore.removeMessage(ack);
				}								
			}

		});
		
		return rc;
	}
	
	private ActiveMQMessage getCacheMessage(MessageIdentity identity) throws JMSException {
		return cacheMessageStore.getMessage(identity);
	}

	/**
	 * 
	 */
	public ActiveMQMessage getMessage(MessageIdentity identity)	throws JMSException {
		ActiveMQMessage answer=null;
		
		Object location = identity.getSequenceNumber();
		if( location==null ) {
			// The sequence number may not have been set but it may still be in the journal.
			synchronized(this) {
				location = addedMessageIds.get(identity);
			}
		}
		// Do we have a Journal sequence number?
		if(location!=null && location instanceof RecordLocation) {
			answer = (ActiveMQMessage) peristenceAdapter.readPacket((RecordLocation)location);
			if( answer !=null )
				return answer;
		}
		
		// If all else fails try the long term message store.
		return longTermStore.getMessage(identity);
	}

	/**
	 * Replays the checkpointStore first as those messages are the oldest ones,
	 * then messages are replayed from the transaction log and then the cache is
	 * updated.
	 * 
	 * @param container
	 * @throws JMSException
	 */
	public synchronized void recover(final QueueMessageContainer container)
			throws JMSException {
		longTermStore.recover(container);
	}

	public void start() throws JMSException {
		longTermStore.start();
	}

	public void stop() throws JMSException {
		longTermStore.stop();
	}

	/**
	 * @return Returns the longTermStore.
	 */
	public MessageStore getLongTermMessageStore() {
		return longTermStore;
	}

	/**
	 * @see org.codehaus.activemq.store.cache.CacheMessageStoreAware#setCacheMessageStore(org.codehaus.activemq.store.cache.CacheMessageStore)
	 */
	public void setCacheMessageStore(CacheMessageStore store) {
		cacheMessageStore = store;
		// Propagate the setCacheMessageStore method call to the longTermStore if possible.
		if( longTermStore instanceof CacheMessageStoreAware ) {
			((CacheMessageStoreAware)longTermStore).setCacheMessageStore(store);
		}
	}

	/**
     * @see org.codehaus.activemq.store.MessageStore#removeAllMessages()
     */
    public void removeAllMessages() throws JMSException {
        peristenceAdapter.checkpoint();
        longTermStore.removeAllMessages();        
    }

}